#!/bin/bash

function error() {
    echo "$0: $*" >&2
    exit 1
}

CLANG=/usr/bin/clang
LLVM_DIS=/usr/bin/llvm-dis
EMSCRIPTEN=$HOME/emscripten/emscripten.py
DIR=`dirname $0`
EZ_DRAW=$DIR

[ -x "$CLANG" ] || error "$CLANG not found or not executable"
[ -x "$LLVM_DIS" ] || error "$LLVM_DIS not found or not executable"
[ -x "$EMSCRIPTEN" ] || error "$EMSCRIPTEN not found or not executable"
[ -d "$EZ_DRAW" ] || error "$EZ_DRAW not a directory"

[ $# == 2 ] || error "$0 <input-c-source> <output-dir>"

INPUT=$1
OUTPUT=$2
NAME=`basename $1 .c`

mkdir -p "$OUTPUT/" || error "could not create $OUTPUT"
#cp "$DIR/ezdraw.js" "$DIR/jquery-1.5.2.min.js" "$OUTPUT/" || error "could not copy support files to $OUTPUT"

JSMIN=./jsmin
#JSMIN=cat # uncomment to disable jsmin

"$CLANG" -m32 -emit-llvm "$INPUT" -o "$OUTPUT/$NAME.o" -c -I "$EZ_DRAW" || error "could not compile program"
"$LLVM_DIS" -show-annotations "$OUTPUT/$NAME.o" -o "$OUTPUT/$NAME.ll" || error "could not generate llvm disassembly"
"$EMSCRIPTEN" -s INCLUDE_FULL_LIBRARY=1 "$OUTPUT/$NAME.ll" | sed 's@^run(args);@//run(args);@' | $JSMIN > "$OUTPUT/$NAME.js" || error "could not generate javascript"
cat "$DIR/template-nosource.html" | sed 's|@@NAME@@|'"$NAME"'|g' > $OUTPUT/$NAME.html || error "could not generate html"

rm "$OUTPUT/$NAME.o" "$OUTPUT/$NAME.ll"
