#!/bin/bash

EZDRAW=./EZ-Draw-1.0
TARGET=EZ-Draw-js
if [ ! -d $EZDRAW ]
then
    echo "$0: cannot find $EZDRAW" >&2
    exit 1
fi

cp $EZDRAW/ez-draw.h .

rm -rf $TARGET
mkdir -p $TARGET
projects="demo0 demo1 demo2 demo3 demo4 demo5 demo6 demo7 demo8a demo8b jeu-bubblet jeu-sudoku jeu-vie jeu-nim jeu-laby jeu-taquin jeu-ezen jeu-heziom jeu-tangram"
for i in $projects
do
    ./build-project.sh $EZDRAW/$i.c $i/
    mv $i $TARGET
done

for i in students/*.c
do
    ./build-project.sh $i $TARGET/`basename $i .c`
done

mkdir $TARGET-src
#build-project.sh  ezdraw.js   gpl-3.0.txt          README                template.html
#ez-draw.h         gen-all.sh  jquery-1.5.2.min.js  sprintf-0.7-beta1.js  template-nosource.html
cp jsmin.c template*.html ez-draw.h *.js *.sh gpl-3.0.txt README Misc-Fixed.ttf $TARGET-src
cp *.js Misc-Fixed.ttf $TARGET
tar zcf $TARGET/$TARGET-src.tgz $TARGET-src
rm -rf $TARGET-src

cat > $TARGET/index.html <<EOF
<html>
<head>
<meta http-equiv="Content-Type" content="text/html; charset=UTF-8"> 
<title>Ez-draw-js</title>
</head>
<body>
<center>
<h1>EZ-Draw-js, pour dessiner sur le web</h1> 
<h4> Copyright (c) 2008-2011 by <br>
Edouard Thiel <a
href="mailto:Edouard.Thiel@lif.univ-mrs.fr?subject=EZ-Draw">&lt;Edouard.Thiel@lif.univ-mrs.fr&gt;</a> 
(langage C)<br> 
and Benoit Favre <a
href="mailto:benoit.favre@lif.univ-mrs.fr?subject=EZ-Draw-js">&lt;benoit.favre@lif.univ-mrs.fr&gt;</a> 
(convertisseur vers javascript) </h4> 
</center> 
<hr>
EZ-Draw-js ("Easy Draw javascript") fonctionne avec
&nbsp;<a
href="http://pageperso.lif.univ-mrs.fr/%7Eedouard.thiel/ez-draw/index.html">EZ-Draw</a> 
(écrit en C par <a
href="http://pageperso.lif.univ-mrs.fr/%7Eedouard.thiel/">Edouard Thiel</a>)
qui facilite
l'écriture de programmes graphiques, avec une gestion
simplifiée des évènements. Il permet de
 créer des fenêtres, y dessiner, réagir à la
souris et au clavier, etc. </p> 
<p>
EZ-Draw-js permet de compiler vos programmes EZ-Draw pour qu'ils fonctionnent sur le web. 
Les sources peuvent être téléchargées <a href="$TARGET-src.tgz">ici</a>.
Attention, un navigateur compatible html5 est requis, comme <a href="http://www.mozilla.com">Firefox 3.5</a> ou <a href="http://www.google.com/chrome">Chrome 6</a>.
</p>
<h3>Les démos EZ-Draw :</h3>
<ul>
<li><a href="demo0/demo0.html">demo0</a> : première fenêtre.</li>
<li><a href="demo1/demo1.html">demo1</a> : fenêtre et évènements.</li>
<li><a href="demo2/demo2.html">demo2</a> : tous les dessins.</li>
<li><a href="demo3/demo3.html">demo3</a> : affichage de texte.</li>
<li><a href="demo4/demo4.html">demo4</a> : trace les évènements.</li>
<li><a href="demo5/demo5.html">demo5</a> : dessin à la souris.</li>
<li><a href="demo6/demo6.html">demo6</a> : plusieurs fenêtres.</li>
<li><a href="demo7/demo7.html">demo7</a> : saisie de texte.</li>
<li><a href="demo8a/demo8a.html">demo8a</a> : animations simples.</li>
<li><a href="demo8b/demo8b.html">demo8b</a> : animations multiples.</li>
</ul>
<h3>Les jeux EZ-Draw :</h3>
<ul>
<li><a href="jeu-bubblet/jeu-bubblet.html">jeu-bubblet</a> : jeu Bubblet (ou Jawbreaker).</li>
<li><a href="jeu-ezen/jeu-ezen.html">jeu-ezen</a> : jeu ezen (puzzle Loops of Zen).</li>
<li><a href="jeu-heziom/jeu-heziom.html">jeu-heziom</a> : jeu Heziom (puzzle hexagonal Hexiom)</li>
<li><a href="jeu-laby/jeu-laby.html">jeu-laby</a> : labyrinthe 3D.</li>
<li><a href="jeu-nim/jeu-nim.html">jeu-nim</a> : le jeu de NIM.</li>
<li><a href="jeu-sudoku/jeu-sudoku.html">jeu-sudoku</a> : jeu de sudoku.</li>
<li><a href="jeu-tangram/jeu-tangram.html">jeu-tangram</a> : jeu de tangram (puzzle).</li>
<li><a href="jeu-taquin/jeu-taquin.html">jeu-taquin</a> : jeu de taquin.</li>
<li><a href="jeu-vie/jeu-vie.html">jeu-vie</a> : jeu de la vie.</li>
</ul>
<h3>Comment ça marche ?</h3>
<p>Le fichier source du programme, écrit en C, est compilé pour la machine virtuelle <a href="http://llvm.org">LLVM</a>, puis converti vers du javascript grâce à <a href="https://github.com/kripken/emscripten/">Emscripten</a>. Enfin, les possibilités du tag html5 canvas sont utilisées pour réaliser l'affichage et gérer les événements.</p>
<h3>Convertissez vos programmes vous-mêmes</h3>
Instructions disponibles prochainement.
<hr>
<h3>Projets des étudiants</h3>
<ul>
<li>L1 2011 :
<ul>
<li><a href="sudo-bonnal/sudo-bonnal.html">Sudoku de François Bonnal</a></li>
<li><a href="sudo-gubernatis/sudo-gubernatis.html">Sudoku de Paul Gubernatis</a></li>
<li><a href="sudo-azzag/sudo-azzag.html">Sudoku de Samir Eddine Azzag</a></li>
<li><a href="sudo-chemin/sudo-chemin.html">Sudoku de Maxime Chemin</a></li>
<li><a href="sudo-delafontaine/sudo-delafontaine.html">Sudoku de David Delafontaine</a></li>
</ul>
</li>
</ul>
<hr> Dernière mise à jour : 9 octobre 2011.
</body>
</html>
EOF
